package com.ym.mec.biz.service.impl;

import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.google.common.collect.Lists;
import com.microsvc.toolkit.middleware.im.ImPluginContext;
import com.microsvc.toolkit.middleware.im.ImPluginService;
import com.microsvc.toolkit.middleware.im.message.GroupMemberWrapper;
import com.ym.mec.auth.api.entity.SysUser;
import com.ym.mec.biz.dal.dao.*;
import com.ym.mec.biz.dal.dto.ImGroupDto;
import com.ym.mec.biz.dal.dto.ImGroupMemberDto;
import com.ym.mec.biz.dal.dto.NameDto;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.enums.EFriendRoleType;
import com.ym.mec.biz.dal.enums.MusicGroupStatusEnum;
import com.ym.mec.biz.dal.enums.school.ESchoolStaffType;
import com.ym.mec.biz.dal.mapper.SchoolStaffMapper;
import com.ym.mec.biz.dal.wrapper.ImGroupWrapper;
import com.ym.mec.biz.service.*;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.entity.ImUserModel;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.common.tenant.TenantContextHolder;
import com.ym.mec.im.ImFeignService;
import com.ym.mec.im.entity.GroupMember;
import com.ym.mec.im.entity.GroupModel;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
@Slf4j
public class ImGroupServiceImpl extends BaseServiceImpl<String, ImGroup> implements ImGroupService {

    @Autowired
    private ImGroupDao imGroupDao;

    @Autowired
    private ImGroupMemberDao imGroupMemberDao;

    @Autowired
    private ImGroupNoticeDao imGroupNoticeDao;

    @Autowired
    private ImGroupMemberService imGroupMemberService;

    @Autowired
    private ImFeignService imFeignService;

    @Autowired
    private SysUserService sysUserService;

    @Autowired
    private ImUserFriendDao imUserFriendDao;

    @Autowired
    private SysConfigService sysConfigService;
    @Autowired
    private ImPluginContext imPluginContext;

    @Autowired
    private CooperationOrganService cooperationOrganService;

    @Autowired
    private MusicGroupDao musicGroupDao;

    @Autowired
    private ClassGroupTeacherMapperDao classGroupTeacherMapperDao;

    @Autowired
    private SchoolStaffMapper schoolStaffMapper;

    @Override
    public BaseDAO<String, ImGroup> getDAO() {
        return imGroupDao;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ImGroup create(String id, Integer userId, String name, String introduce, String memo, String tags, String img, String type, ImGroup.GroupTypeEnum groupType) {
        ImGroup imGroup = imGroupDao.get(id);
        if (imGroup != null) {
            return imGroup;
        }

        imGroup = new ImGroup();

        Date date = new Date();

        imGroup.setId(id);
        imGroup.setCreateTime(date);
        imGroup.setIntroduce(introduce);

        if(userId != null){
            imGroup.setMemberNum(1);
        }else{
            imGroup.setMemberNum(0);
        }
        imGroup.setMemo(memo);
        imGroup.setName(name);
        imGroup.setTags(tags);
        imGroup.setImg(img);
        imGroup.setType(type);
        imGroup.setGroupType(groupType);
        imGroup.setUpdateTime(date);
        imGroupDao.insert(imGroup);

        String groupId = imGroup.getId();

        if(userId != null){
            imGroupMemberService.join(id, userId, null, true);

            GroupMember groupMember = new GroupMember(userId.toString(), groupId);

            imFeignService.groupCreate(new GroupModel(groupId, new GroupMember[] { groupMember }, name));
        }else{
            imFeignService.groupCreate(new GroupModel(groupId, null, name));
        }

        return imGroup;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String createGroup(ImGroupDto imGroupDto){
        List<String> userIdList = imGroupDto.getUserIdList();
        Integer currentUserId = null;
        if (CollectionUtils.isEmpty(userIdList)){
            userIdList.add(sysUserService.getUserId().toString());
        }
        String groupName = imGroupDto.getGroupName();
        String type = imGroupDto.getType();
        Optional.ofNullable(groupName).filter(StringUtils::isNotBlank).orElseThrow(() -> new BizException("群名称不能为空"));
        Optional.ofNullable(type).filter(StringUtils::isNotBlank).orElseThrow(() -> new BizException("群类型不能为空"));
        Date date = new Date();
        ImGroup imGroup = new ImGroup();
        String groupId = UUID.randomUUID().toString();
        imGroup.setId(groupId);
        imGroup.setCreateTime(date);
        imGroup.setIntroduce(groupName);
        imGroup.setMemo(groupName);
        imGroup.setMemberNum(userIdList.size());
        imGroup.setName(groupName);
        imGroup.setType(type);
        imGroup.setGroupType(imGroupDto.getGroupType());
        imGroup.setUpdateTime(date);
        imGroupDao.insert(imGroup);
        List<ImGroupMember> groupMemberList = imGroupMemberDao.queryMembers(groupId, userIdList, TenantContextHolder.getTenantId());
        if(currentUserId != null){
            groupMemberList.get(0).setRoleType("乐团主管");
        }
        imGroupMemberDao.batchInsert(groupMemberList);
        Function<String,GroupMember> newGroup = userId ->new GroupMember(userId, groupId);
        List<GroupMember> groupMembers =  userIdList.stream().map(newGroup).collect(Collectors.toList());
        GroupMember[] members = new GroupMember[groupMembers.size()];
        groupMembers.toArray(members);
        imFeignService.groupCreate(new GroupModel(groupId,members,groupName));
        return groupId;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean cancel(String imGroupId) {
        // 删除群成员
        imGroupMemberDao.deleteByImGroupId(imGroupId);
        // 删除群公告
        imGroupNoticeDao.deleteByImGroupId(imGroupId);
        // 删除群信息
        imGroupDao.delete(imGroupId);

        imFeignService.groupDismiss(new GroupModel(imGroupId, null));

        return true;
    }

    @Override
    public List<ImGroup> queryByUserId(Integer userId, String search,String groupType) {
        return imGroupDao.queryByUserId(userId, search,groupType);
    }

    @Override
    public List<ImGroupMemberDto> queryMemberById(String imGroupId) {
        return  queryMemberById(imGroupId,null);
    }


    @Override
    public List<ImGroupMemberDto> queryMemberById(String imGroupId,String search) {
        List<ImGroupMemberDto> imGroupMemberDtos = imGroupDao.queryMemberById(imGroupId,search);

        for (ImGroupMemberDto imGroupMemberDto : imGroupMemberDtos) {
            imGroupMemberDto.setType(EFriendRoleType.getByName(imGroupMemberDto.getRoleType()).name());
            imGroupMemberDto.getUser().setUsername(imGroupMemberDto.getGroupNickname());
            imGroupMemberDto.setNickname(imGroupMemberDto.getGroupNickname());
        }
        return imGroupMemberDtos;
    }

    @Override
    public ImGroupMemberDto queryMember(String imGroupId, Integer userId) {
        ImGroupMemberDto imGroupMemberDto = imGroupDao.queryMember(imGroupId, userId);
        if(Objects.isNull(imGroupMemberDto)){
            imGroupMemberDto = imGroupDao.queryMember(null, userId);
            if (Objects.nonNull(imGroupMemberDto)){
                imGroupMemberDto.setRoleType(null);
            }
        }
        return imGroupMemberDto;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int updateNickName(Integer userId, String nickName) {
        //修改群成员备注
        int i = imGroupDao.updateNickname(userId, nickName);
        imGroupDao.updateUserFriendNickname(userId, nickName);
        return i;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateNickName1(Integer userId, String nickName) {
        //修改群成员备注
        imGroupDao.updateNickname(userId, nickName);
        imGroupDao.updateUserFriendNickname(userId, nickName);
        //修改sysUser名称，如果包含学员的角色，那么修改userName。否则修改realName
        SysUser sysUser = sysUserService.queryUserById(userId);
        if(sysUser.getUserType().contains("STUDENT")){
            sysUser.setUsername(nickName);
        }else {
            sysUser.setRealName(nickName);
        }
        sysUserService.updateSysUser(sysUser);
        //同步融云基本信息
        imFeignService.update(new ImUserModel(sysUser.getId().toString(),nickName,sysUser.getAvatar()));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateImGroup(ImGroup imGroup) {
        imGroup.setMemberNum(null);
        imGroupDao.update(imGroup);
        imFeignService.groupUpdate(new GroupModel(imGroup.getId(),null,imGroup.getName()));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void dismissGroup(String imGroupId) {
        imFeignService.groupDismiss(new GroupModel(imGroupId, null));
        imGroupDao.delete(imGroupId);
        imGroupMemberDao.deleteByImGroupId(imGroupId);
        imGroupNoticeDao.deleteByImGroupId(imGroupId);
    }

    @Override
    public List<NameDto> queryDetail(List<NameDto> nameDto) {
        List<NameDto> dtos = new ArrayList<>();
        if(CollectionUtils.isNotEmpty(nameDto)){
            Map<Integer, List<NameDto>> collect = nameDto.stream().collect(Collectors.groupingBy(NameDto::getType));
            for (Integer integer : collect.keySet()) {
                List<NameDto> nameDtos = collect.get(integer);
                if(CollectionUtils.isNotEmpty(nameDtos)){
                    String idList = nameDtos.stream().map(e -> e.getId()).collect(Collectors.joining(","));
                    if(integer.equals(3)){
                        dtos.addAll(imGroupDao.queryNameByIds(idList));
                    }else if (integer.equals(1)){
                        Integer userId = sysUserService.getUserId();
                        dtos.addAll(imUserFriendDao.queryNameByIds(idList,userId));
                    }
                }
            }
        }
        return dtos;
    }

    @Override
    public List<ImGroup> queryByUserIdV2(Integer userId, String search, String groupType, String musicGroupId, String classType) {
        return imGroupDao.queryByUserIdV2(userId, search, groupType, musicGroupId, classType);
    }

    @Override
    public List<ImGroupMemberDto> queryMemberByIdV2(String imGroupId, String search, Integer subjectId, Boolean vipFlag) {
        return imGroupDao.queryMemberByIdV2(imGroupId,search,subjectId,vipFlag);
    }

    /**
     * 用户退出所有群
     * <p>
     * 删除时判断是否群主，
     * 如果是，则随机分配一个群主，
     * 如果删除的是最后一个群成员，那么解散群
     * @param userId 用户id
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void quitAllGroup(Long userId) {

        // 退出群组
        String serviceProvider = sysConfigService.findByParamName(SysConfigService.IM_SERVICE_PROVIDER).getParanValue();

        ImPluginService imPluginService =imPluginContext.getPluginService(serviceProvider);

        List<ImGroupMember> imGroupMembers = imGroupMemberDao.queryMembersByUserId(userId);

        if (CollectionUtils.isEmpty(imGroupMembers)) {
            return;
        }
        for (ImGroupMember imGroupMember : imGroupMembers) {
            // 如果是群主，随机分配一个群主
            if (imGroupMember.getIsAdmin()) {
                try {
                    // 获取一个群成员
                    ImGroupMember imGroupMember1 = imGroupMemberDao.queryOneByGroupIdAndNotUser(imGroupMember.getImGroupId(), userId);
                    if (Objects.nonNull(imGroupMember1)) {
                        groupChangeOwner(imGroupMember1.getUserId(),imGroupMember.getImGroupId());
                    }
                } catch (Exception e) {
                    log.error("更换群主失败{},{}", userId, imGroupMember.getImGroupId(), e);
                }
            }
            try {
                imPluginService.groupQuit(imGroupMember.getImGroupId(), GroupMemberWrapper.ImGroupMember.builder().userId(userId).imUserIdFormat(false).build());
            } catch (Exception e) {
                log.error("用户退出群失败{},{}", userId, imGroupMember.getImGroupId(), e);
            }
        }
        List<String> imGroupIds = imGroupMembers.stream().map(ImGroupMember::getImGroupId).collect(Collectors.toList());
        // 群组没人，删除群组
        List<ImGroup> imGroups = imGroupDao.queryByUserId(userId.intValue(),null, ImGroup.GroupTypeEnum.SCHOOL.getCode());


        imGroupMemberDao.deleteByUserId(userId);
        // 更新群组人员数
        imGroupDao.addMemberNum(imGroupIds,-1);

        if (CollectionUtils.isEmpty(imGroups)) {
            return;
        }


        List<ImGroup> groups = imGroups.stream().filter(imGroup -> imGroup.getMemberNum().equals(1)).collect(Collectors.toList());
        for (ImGroup group : groups) {
            try {
                GroupMemberWrapper.ImGroupMember build = GroupMemberWrapper.ImGroupMember.builder().userId(userId).imUserIdFormat(false).build();
                imPluginService.groupDismiss(group.getId(), Lists.newArrayList(build));
            } catch (Exception e) {
                log.error("解散群组失败{},{}", userId, group.getId(), e);
            }
            imGroupDao.delete(group.getId());
        }
    }

    /**
     * 用户加入群
     *
     * @param schoolId 学校id
     * @param userId   用户id
     * @param schoolStaffType   是否是群主
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void addSchoolStaffGroup(Long schoolId, Integer userId,String name,String avatar, ESchoolStaffType schoolStaffType) throws Exception {

        // 查询群是否存在
        ImGroup imGroup = imGroupDao.queryBySchoolId(schoolId);

        String serviceProvider = sysConfigService.findByParamName(SysConfigService.IM_SERVICE_PROVIDER).getParanValue();

        ImPluginService imPluginService =imPluginContext.getPluginService(serviceProvider);


        // 注册主播用户信息到三方平台
        imPluginService.register(userId.toString(), name, avatar);

        // 群组不存在，先创建群组
        if (Objects.isNull(imGroup)) {
            CooperationOrgan cooperationOrgan = cooperationOrganService.get(schoolId.intValue());
            imGroup = new ImGroup();
            imGroup.setSchoolId(schoolId);
            imGroup.setId(IdWorker.getIdStr());
            imGroup.setName("教学团队衔接群");
            imGroup.setIntroduce(imGroup.getName());
            imGroup.setMemo(imGroup.getName());
            imGroup.setMemberNum(0);
            imGroup.setImg(sysConfigService.findByParamName(SysConfigService.SCHOOL_IM_GROUP_IMG).getParanValue());
            imGroup.setType(ImGroup.GroupTypeEnum.SCHOOL.getCode());
            imGroup.setGroupType(ImGroup.GroupTypeEnum.SCHOOL);
            imGroup.setCreateTime(new Date());
            imGroup.setUpdateTime(new Date());
            imGroup.setTenantId(cooperationOrgan.getTenantId());
            imGroupDao.insert(imGroup);
            imPluginService.groupCreate(imGroup.getId(), imGroup.getName(), userId.toString());

            initSchoolGroup(imGroup.getSchoolId());
        }
        imPluginService.groupJoin(imGroup.getId(), imGroup.getName(), GroupMemberWrapper.ImGroupMember.builder()
                .userId(userId.longValue()).imUserIdFormat(false).build());

        if (schoolStaffType.equals(ESchoolStaffType.ORCHESTRA_LEADER)) {
            groupChangeOwner(userId, imGroup.getId());
        }
        ImGroupMember imGroupMember = new ImGroupMember();
        imGroupMember.setImGroupId(imGroup.getId());
        imGroupMember.setUserId(userId);
        imGroupMember.setIsAdmin(schoolStaffType.equals(ESchoolStaffType.ORCHESTRA_LEADER));
        imGroupMember.setRoleType(schoolStaffType.getDescribe());
        imGroupMember.setType(EFriendRoleType.getByName(schoolStaffType.getDescribe()).name());
        imGroupMember.setTenantId(imGroup.getTenantId());
        imGroupMemberDao.insert(imGroupMember);

        updateImGroupUserNumber(imGroup.getId());
    }

    private void initSchoolGroup(Long schoolId) {


        // 查询乐团主管
        List<MusicGroup> musicGroupList = musicGroupDao.findByCooperationId(schoolId.intValue());
        if (CollectionUtils.isEmpty(musicGroupList)) {
            return;
        }
        musicGroupList = musicGroupList.stream()
                .filter(o -> o.getStatus().equals(MusicGroupStatusEnum.PROGRESS))
                .collect(Collectors.toList());
        if (CollectionUtils.isEmpty(musicGroupList)) {
            return;
        }

        // 乐团主管
        List<Integer> groupLeaderIds = musicGroupList.stream().map(MusicGroup::getEducationalTeacherId).collect(Collectors.toList());

        // 乐团ID集合
        List<String> musicGroupIds = musicGroupList.stream().map(MusicGroup::getId).collect(Collectors.toList());

        // 班级列表主教/助教老师
        List<ClassGroupTeacherMapper> teacherMapperList = classGroupTeacherMapperDao.findAllByMusicGroupIds(musicGroupIds);

        List<Integer> teacherIds = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(teacherMapperList)) {
            teacherIds.addAll(teacherMapperList.stream().map( o -> o.getUserId()).collect(Collectors.toSet()));
        }

        List<GroupMemberWrapper.ImGroupMember> imGroupMembers = new ArrayList<>();
        for (Integer groupLeaderId : groupLeaderIds) {
            imGroupMembers.add(GroupMemberWrapper.ImGroupMember.builder()
                    .userId(groupLeaderId.longValue()).roleType(EFriendRoleType.ORCHESTRA_MANAGER.getMsg()).imUserIdFormat(false).build());
        }

        for (Integer teacherId : teacherIds) {
            imGroupMembers.add(GroupMemberWrapper.ImGroupMember.builder()
                    .userId(teacherId.longValue()).roleType(EFriendRoleType.ORCHESTRA_TEACHER.getMsg()).imUserIdFormat(false).build());
        }

        addSchoolGroup(schoolId, imGroupMembers);
    }

    /**
     * 加入学校群聊
     *
     * @param schoolId
     * @param imGroupMembers
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void addSchoolGroup(Long schoolId, List<GroupMemberWrapper.ImGroupMember> imGroupMembers) {
        // 添加乐团领队，班级老师
        List<ImGroupMember> imGroupMemberList =new ArrayList<>();


        // 查询群是否存在
        ImGroup imGroup = imGroupDao.queryBySchoolId(schoolId);
        if (imGroup == null) {
            return;
        }
        if (CollectionUtils.isEmpty(imGroupMembers)) {
            return;
        }

        for (GroupMemberWrapper.ImGroupMember groupMember : imGroupMembers) {
            ImGroupMember imGroupMember = new ImGroupMember();
            imGroupMember.setImGroupId(imGroup.getId());
            imGroupMember.setUserId(groupMember.getUserId().intValue());
            imGroupMember.setIsAdmin(false);
            imGroupMember.setRoleType(groupMember.getRoleType());
            imGroupMember.setType(EFriendRoleType.getByName(groupMember.getRoleType()).name());
            imGroupMember.setTenantId(imGroup.getTenantId());
            imGroupMemberList.add(imGroupMember);
            groupMember.setImUserIdFormat(false);
        }

        String serviceProvider = sysConfigService.findByParamName(SysConfigService.IM_SERVICE_PROVIDER).getParanValue();

        ImPluginService imPluginService =imPluginContext.getPluginService(serviceProvider);

        try {
            imPluginService.groupJoin(imGroup.getId(), imGroup.getName(), imGroupMembers);
        } catch (Exception e) {
            log.error("添加群成员失败{}", imGroup.getId(), e);
        }
        imGroupMemberDao.batchInsert(imGroupMemberList);
        updateImGroupUserNumber(imGroup.getId());
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public void quitSchoolGroup(Long schoolId, List<GroupMemberWrapper.ImGroupMember> imGroupMembers) {
        // 添加乐团领队，班级老师


        // 查询群是否存在
        ImGroup imGroup = imGroupDao.queryBySchoolId(schoolId);
        if (imGroup == null) {
            return;
        }

        if (CollectionUtils.isEmpty(imGroupMembers)) {
            return;
        }


        String serviceProvider = sysConfigService.findByParamName(SysConfigService.IM_SERVICE_PROVIDER).getParanValue();

        List<Integer> userIds = new ArrayList<>();
        ImPluginService imPluginService =imPluginContext.getPluginService(serviceProvider);
        for (GroupMemberWrapper.ImGroupMember imGroupMember : imGroupMembers) {
            imGroupMember.setImUserIdFormat(false);
            userIds.add(imGroupMember.getUserId().intValue());
        }
        try {
            imPluginService.groupQuit(imGroup.getId(),imGroupMembers);
        } catch (Exception e) {
            log.error("退出群成员失败{}", imGroup.getId(), e);
        }


        imGroupMemberDao.delByImGroupAndUserId(imGroup.getId(),userIds);
        updateImGroupUserNumber(imGroup.getId());
    }

    private void updateImGroupUserNumber(String imGroupId) {
        ImGroup imGroup = new ImGroup();
        imGroup.setId(imGroupId);
        imGroup.setMemberNum(imGroupMemberDao.getByGroupId(Lists.newArrayList(imGroupId)).size());
        imGroupDao.updateMember(imGroup.getId(),imGroup.getMemberNum());
    }

    /**
     * 群主移交
     * @param userId 用户id
     * @param imGroupId     群组id
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean groupChangeOwner(Integer userId, String imGroupId) throws Exception {

        String serviceProvider = sysConfigService.findByParamName(SysConfigService.IM_SERVICE_PROVIDER).getParanValue();

        ImPluginService imPluginService =imPluginContext.getPluginService(serviceProvider);

        ImGroupMember imGroupMember = imGroupMemberDao.getAdmin(imGroupId);
        Integer adminId = userId;
        if (Objects.nonNull(imGroupMember)) {
            adminId = imGroupMember.getUserId();
            imGroupMember.setIsAdmin(false);
            imGroupMemberDao.update(imGroupMember);
        }
        imPluginService.changeGroupOwner(imGroupId, userId.toString(), adminId.toString());

        imGroupMemberDao.updateAdmin(imGroupId, userId);
        return true;
    }

    /**
     * 加群
     *
     * @param imGroupIds    群组ID
     * @param userId          用户信息
     * @param roleType      用户类型
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void addImGroup(List<String> imGroupIds, Long  userId, String roleType) {
        if (CollectionUtils.isEmpty(imGroupIds)) {
            return;
        }

        String serviceProvider = sysConfigService.findByParamName(SysConfigService.IM_SERVICE_PROVIDER).getParanValue();

        ImPluginService imPluginService =imPluginContext.getPluginService(serviceProvider);

        List<ImGroup> groupList = imGroupDao.getByIds(imGroupIds);

        // id 分组
        Map<String, ImGroup> map = groupList.stream().collect(Collectors.toMap(ImGroup::getId, Function.identity()));


        for (String musicGroupId : imGroupIds) {
            ImGroup imGroup = map.get(musicGroupId);
            if (imGroup == null) {
                continue;
            }
            try {
                imPluginService.groupJoin(imGroup.getId(), imGroup.getName(), GroupMemberWrapper.ImGroupMember.builder()
                        .userId(userId)
                        .imUserIdFormat(false).build());

                ImGroupMember imGroupMember = new ImGroupMember();
                imGroupMember.setImGroupId(imGroup.getId());
                imGroupMember.setUserId(userId.intValue());
                imGroupMember.setIsAdmin(false);
                imGroupMember.setRoleType(roleType);
                imGroupMember.setType(EFriendRoleType.getByName(roleType).name());
                imGroupMember.setTenantId(imGroup.getTenantId());
                imGroupMemberDao.insert(imGroupMember);

                updateImGroupUserNumber(imGroup.getId());
            } catch (Exception e) {
                log.error("加入群聊失败",e);
            }

        }
    }

    /**
     * 查询群组
     *
     * @param query 查询条件
     * @return 群组列表
     */
    @Override
    public List<ImGroup> queryByUserId(ImGroupWrapper.ImQuery query) {
        return imGroupDao.getByUserId(query);
    }

    /**
     * 退出群组
     *
     * @param imGroupIds 群组ID
     * @param userId     用户ID
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void quitGroup(List<String> imGroupIds, Integer userId) {
        if (CollectionUtils.isEmpty(imGroupIds)) {
            return;
        }

        String serviceProvider = sysConfigService.findByParamName(SysConfigService.IM_SERVICE_PROVIDER).getParanValue();

        ImPluginService imPluginService =imPluginContext.getPluginService(serviceProvider);

        List<ImGroup> groupList = imGroupDao.getByIds(imGroupIds);

        List<com.ym.mec.biz.dal.entity.ImGroupMember> imGroupMemberList = new ArrayList<>();
        for (ImGroup imGroup : groupList) {
            try {
                imPluginService.groupQuit(imGroup.getId(),  GroupMemberWrapper.ImGroupMember.builder().userId(userId.longValue()).imUserIdFormat(false).build());
            } catch (Exception e) {
                log.error("退出群聊失败",e);
            }
            ImGroupMember imGroupMember = new ImGroupMember();
            imGroupMember.setImGroupId(imGroup.getId());
            imGroupMember.setUserId(userId);
            imGroupMemberList.add(imGroupMember);
        }

        imGroupMemberService.batchDeleteByGroupIdAndUserId(imGroupMemberList);
        for (String imGroupId : imGroupIds) {
            updateImGroupUserNumber(imGroupId);
        }
    }


    /**
     * 学校人员加入群组
     *
     * @param cooperationOrganId 合作机构id
     * @param imGroupId 群组id
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void schoolJoinGroup(Integer cooperationOrganId, String imGroupId) {

        CooperationOrgan cooperationOrgan = cooperationOrganService.get(cooperationOrganId);
        if (cooperationOrgan == null) {
            return;
        }
        List<SchoolStaff> schoolStaffs = schoolStaffMapper.getByCoopIdAndType(cooperationOrganId, ESchoolStaffType.ORCHESTRA_LEADER);
        if (CollectionUtils.isEmpty(schoolStaffs)) {
            return;
        }


        // 查询群是否存在
        ImGroup imGroup = imGroupDao.get(imGroupId);
        if (imGroup == null) {
            return;
        }


        SchoolStaff schoolStaff = schoolStaffs.get(0);



        addImGroup(Lists.newArrayList(imGroup.getId()),schoolStaff.getUserId(),ESchoolStaffType.ORCHESTRA_LEADER.getDescribe());

    }
}