package com.ym.mec.web.controller.school;


import com.ym.mec.biz.dal.dto.StudentLessonExaminationDetailDto;
import com.ym.mec.biz.dal.dto.StudentLessonExaminationDto;
import com.ym.mec.biz.dal.dto.StudentLessonTrainingDetailWrapper;
import com.ym.mec.biz.dal.entity.StudentCourseHomework;
import com.ym.mec.biz.dal.enums.ELessonTrainingType;
import com.ym.mec.biz.dal.page.StudentLessonExaminationQueryDto;
import com.ym.mec.biz.dal.page.StudentLessonExaminationQueryInfo;
import com.ym.mec.biz.service.StudentCourseHomeworkService;
import com.ym.mec.biz.service.StudentLessonExaminationDetailService;
import com.ym.mec.biz.service.StudentLessonExaminationService;
import com.ym.mec.common.controller.BaseController;
import com.ym.mec.common.entity.HttpResponseResult;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;
import java.util.Objects;

@Api(tags = "学校学员课后作业")
@RestController
@RequestMapping("/schoolStudentHomework")
public class SchoolStudentHomeworkController extends BaseController {

    @Resource
    private StudentLessonExaminationService studentLessonExaminationService;
    @Resource
    private StudentLessonExaminationDetailService studentLessonExaminationDetailService;

    @Autowired
    private StudentCourseHomeworkService studentCourseHomeworkService;

    @ApiOperation(value = "获取课后评测学员列表")
    @PostMapping("queryLessonStudent")
    public HttpResponseResult<PageInfo<StudentLessonExaminationDto>> queryLessonStudent(@RequestBody StudentLessonExaminationQueryInfo queryInfo) {
        return succeed(studentLessonExaminationService.queryPage(queryInfo));
    }

    @ApiOperation(value = "获取单个学员评测")
    @PostMapping("getLessonStudent")
    public HttpResponseResult<StudentLessonExaminationDto> getLessonStudent(@RequestBody StudentLessonExaminationQueryInfo queryInfo) {
        return succeed(studentLessonExaminationService.queryPage(queryInfo).getRows().get(0));
    }

    @ApiOperation(value = "获取学员评测详情列表")
    @PostMapping("queryStudentLessonDetail")
    public HttpResponseResult<List<StudentLessonExaminationDetailDto>> queryStudentLessonDetail(@RequestBody StudentLessonExaminationQueryDto query) {
        return succeed(studentLessonExaminationDetailService.queryAll(query));
    }


    @ApiOperation(value = "根据课程计划获取需要交作业的学生-公用")
    @PostMapping("/findCourseStudentsPublic/v2")
    public HttpResponseResult<List<StudentCourseHomework>> findCourseStudentsPublicV2(@Validated @RequestBody StudentLessonTrainingDetailWrapper.StudentLessonTrainingQuery query){
        if(ELessonTrainingType.HOMEWORK.equals(query.getType())){
            if (Objects.isNull(query.getCourseScheduleId())) {
                throw new BizException("请指定课程");
            }
            return succeed(studentCourseHomeworkService.findStudentCourseHomeworkByCourseV2(query));
        }else{
            return succeed(studentCourseHomeworkService.findExtraExerciseStudentsV2(query));
        }
    }

    @ApiOperation(value = "查询单个学生的训练")
    @GetMapping("/findCourseHomeworkStudentDetailPublic/v2")
    public HttpResponseResult<StudentLessonTrainingDetailWrapper.StudentLessonTraining> findCourseHomeworkStudentDetailPublicV2(@RequestParam Long courseScheduleId, @RequestParam Long userId, @RequestParam ELessonTrainingType type){
        if(ELessonTrainingType.HOMEWORK.equals(type)){

            // 获取学生作业界面详细信息
            return succeed(studentCourseHomeworkService.findCourseHomeworkStudentDetailPublicV2(courseScheduleId, userId,type));
        }else{

            return succeed(studentCourseHomeworkService.findExtracurricularExercisesDetailPublicV2(courseScheduleId, userId, type));
        }
    }
}

